package de.knauf.eclipse.jbossdeployer.jsr88;

import java.io.File;

import javax.enterprise.deploy.shared.ModuleType;
import javax.enterprise.deploy.shared.StateType;
import javax.enterprise.deploy.spi.DeploymentManager;
import javax.enterprise.deploy.spi.Target;
import javax.enterprise.deploy.spi.TargetModuleID;
import javax.enterprise.deploy.spi.status.DeploymentStatus;
import javax.enterprise.deploy.spi.status.ProgressObject;

/**
 * JSR88-Undeployer for JBoss 4.2/JBoss 5.0
 * 
 * @author Wolfgang Knauf
 * 
 */
public class JSR88Undeployer
{
  /**
   * Main method. Waits until an app is deployed.
   * 
   * @param args Command line. <br>
   *          first argument: the url of the deployed file. <br>
   *          second argument: server host ("locahost") <br>
   *          third argument: port (default: 1099) <br> 
   */
  public static void main(String[] args)
  {
    try
    {
      if (args.length != 3)
      {
        throw new IllegalArgumentException("Three arguments must be specified instead of " + args.length + ": url serverAddress jndiPort");
      }
      // Get file:
      String strUrl = args[0];
      File fileDeploy = new File(strUrl);
      System.out.println("Undeploying file '" + strUrl + "'");

      // Get a Deployment manager:
      DeploymentManager dm = JSR88DeployerUtil.getDeploymentManager(args[1], args[2]);
      if (dm == null)
      {
        return;
      }

      // Find the module for the file:
      Target[] targets = dm.getTargets();
      //Check ModuleType depending on the file extension:
      String strFileName = fileDeploy.getName();
      ModuleType moduleType = null;

      if (strFileName.toLowerCase().endsWith(".ear"))
      {
        moduleType = ModuleType.EAR;
      }
      else if (strFileName.toLowerCase().endsWith(".jar"))
      {
        moduleType = ModuleType.EJB;
      }
      else if (strFileName.toLowerCase().endsWith(".war"))
      {
        moduleType = ModuleType.WAR;
      }
      else
      {
        System.out.println("Unknown extension of file: " + strFileName);
        return;
      }

      TargetModuleID[] modules = dm.getAvailableModules(moduleType, targets);
      // Module list may be null if nothing is deployed.
      if (modules != null)
      {
        TargetModuleID moduleIDUndeploy = null;
        for (int intModule = 0; intModule < modules.length && moduleIDUndeploy == null; intModule++)
        {
          //Debugging output:
          //System.out.println("Module " + intModule + ": " + modules[intModule] + ", ModuleID: " + modules[intModule].getModuleID());
          if (modules[intModule].getModuleID().endsWith(strFileName) == true)
          {
            // We found a module:
            moduleIDUndeploy = modules[intModule];
          }
        }

        // Undeploy file if found.
        if (moduleIDUndeploy != null)
        {
          JSR88Undeployer.jsr88Undeploy(new TargetModuleID[] { moduleIDUndeploy }, dm);
          System.out.println ("Module with ID '" + moduleIDUndeploy.toString() + "' undeployed");
        }
        else
        {
          System.out.println ("File " + strFileName + " not deployed");
        }
      }
      else
      {
        //If nothing is deployed, the module list is empty!
        System.out.println ("File " + strFileName + " not deployed");
      }
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /**Undeploys a list of modules from the server.
   * 
   * @param resultTargetModuleIDs An array of module IDs
   * @param dm  The DeploymentManager used to communicate with the server
   * @throws Exception Any errors....
   */
  private static void jsr88Undeploy(TargetModuleID[] resultTargetModuleIDs, DeploymentManager dm) throws Exception
  {
    // Step 1: stop application:
    ProgressObject progress = dm.stop(resultTargetModuleIDs);
    DeploymentStatus status = progress.getDeploymentStatus();
    JSR88DeployerUtil.waitForCompletion(status);
    
    // Check that the app is stopped:
    if (StateType.COMPLETED != status.getState())
    {
      System.out.println("Progress.getState != COMPLETED: " + status.getMessage());
      return;
    }
  
    // Step 2: undeploy:
    progress = dm.undeploy(resultTargetModuleIDs);
    status = progress.getDeploymentStatus();
    JSR88DeployerUtil.waitForCompletion(status);
    //Check the app is undeployed:
    if (StateType.COMPLETED != status.getState())
    {
      System.out.println("Progress.getState != COMPLETED: " + status.getMessage());
    }
  }

}
